# Generated by roxygen2: do not edit by hand

S3method(as_rdf,data.frame)
S3method(as_rdf,list)
S3method(c,rdf)
S3method(format,rdf)
S3method(length,rdf)
S3method(print,rdf)
S3method(write_nquads,data.frame)
S3method(write_nquads,rdf)
export(as_rdf)
export(rdf)
export(rdf_add)
export(rdf_free)
export(rdf_has_bdb)
export(rdf_parse)
export(rdf_query)
export(rdf_serialize)
export(read_nquads)
export(write_nquads)
importClassesFrom(redland,Model)
importClassesFrom(redland,Parser)
importClassesFrom(redland,Query)
importClassesFrom(redland,Serializer)
importClassesFrom(redland,Statement)
importClassesFrom(redland,Storage)
importClassesFrom(redland,World)
importFrom(dplyr,left_join)
importFrom(methods,new)
importFrom(readr,read_csv)
importFrom(redland,is.null.externalptr)
importFrom(redland,librdf_model_size)
importFrom(redland,librdf_query_results_to_string2)
importFrom(stringi,stri_unescape_unicode)
importFrom(tidyr,gather)
importFrom(utils,capture.output)
importFrom(utils,download.file)
importFrom(utils,write.table)
importMethodsFrom(redland,addStatement)
importMethodsFrom(redland,executeQuery)
importMethodsFrom(redland,freeModel)
importMethodsFrom(redland,freeParser)
importMethodsFrom(redland,freeQuery)
importMethodsFrom(redland,freeQueryResults)
importMethodsFrom(redland,freeSerializer)
importMethodsFrom(redland,freeStatement)
importMethodsFrom(redland,freeStorage)
importMethodsFrom(redland,freeWorld)
importMethodsFrom(redland,parseFileIntoModel)
importMethodsFrom(redland,serializeToFile)
importMethodsFrom(redland,setNameSpace)
