RDFLib is a Python library for working with RDF, a simple yet 
powerful language for representing information.

The library contains parsers and serializers for RDF/XML, N3,
NTriples, Turtle, TriX and RDFa.

The library presents a Graph interface which can be backed by
any one of a number of Store implementations. 

This core RDFLib package includes store implementations for 
in memory storage and persistent storage on top of the Berkeley DB. 

RDFLib is open source and is maintained in a 
`GitHub <http://github.com/RDFLib/rdflib/>`_ repository. RDFLib releases, current and previous 
are listed on `PyPi <pypi.python.org/pypi/rdflib/>`_

RDFLib may be easy_installed::

    $ easy_install rdflib

More information is available on the project webpage:

https://github.com/RDFLib/rdflib/

The RDFExtras project offers several additional stores as well as a 
SPARQL engine for use with RDFLib: 

https://github.com/RDFLib/rdfextras/

The documentation can be built by doing::

    $ python setup.py build_sphinx

And is also available from ReadTheDocs: 

http://readthedocs.org/docs/rdflib/

Support is available through the rdflib-dev group: 

http://groups.google.com/group/rdflib-dev

and on the IRC channel #redfoot on the freenode.net server

