Some tools built using rdflib. Eventually these will become part of a
release separate from rdflib, but until such a separate release exists
we'll start gathering some here.

These rdflib_tool are being released under than same license as
rdflib.