This README is for the W3C RDF Working Group's Turtle test suite.
This test suite contains four kinds of tests:

  132 Evaluation (rdft:TestTurtleEval) - a pair of an input turtle
  file and reference ntriples file.

  77 Positive syntax (rdft:TestTurtlePositiveSyntax) - an input turtle
  file with no syntax errors.

  78 Negative syntax (rdft:TestTurtleNegativeSyntax) - an input turtle
  file with at least one syntax error.

  4 Negative Evaluation (rdft:TestTurtleNegativeEval) - a pair of an
  input turtle file and reference ntriples file. These tests have the
  same properties as rdft:TestTurtleNegativeSyntax.

The manifest.ttl file in this directory lists all of the tests in the
RDF WG's Turtle test suite. Each test is one of the above tests. All
tests have a name (mf:name) and an input (mf:action). The Evaluation
tests have an expected result (mf:result).

• An implementation passes an Evaluation test if it parses the input
  into a graph, parses the expecte result into another graph, and
  those two graphs are isomorphic (see
  <http://www.w3.org/TR/rdf11-concepts/#graph-isomorphism>).

• An implementation passes a positive syntax test if it parses the
  input.

• An implementation passes a negative syntax test if it fails to parse
  the input.


RELATIVE IRI RESOLUTION:

The home of the test suite is <https://w3c.github.io/N3/tests/TurtleTests/>.
Per RFC 3986 section 5.1.3, the base IRI for parsing each file is the
retrieval IRI for that file. For example, the tests turtle-subm-01 and
turtle-subm-27 require relative IRI resolution against a base of
<https://w3c.github.io/N3/tests/TurtleTests/turtle-subm-01.ttl> and
<https://w3c.github.io/N3/tests/TurtleTests/turtle-subm-27.ttl> respectively.


CHARACTER ENCODING:

The Turtle language uses UTF-8 encoding. The following tests include
non-ascii characters:
  localName_with_assigned_nfc_bmp_PN_CHARS_BASE_character_boundaries
  localName_with_assigned_nfc_PN_CHARS_BASE_character_boundaries *
  localName_with_nfc_PN_CHARS_BASE_character_boundaries *
  labeled_blank_node_with_PN_CHARS_BASE_character_boundaries *
  LITERAL1_with_UTF8_boundaries *
  LITERAL_LONG1_with_UTF8_boundaries *
  LITERAL2_with_UTF8_boundaries *
  LITERAL_LONG2_with_UTF8_boundaries *

Those marked with a * include characters with codepoints greater than
U+FFFD and are thus expressed as a pair of surrogate characters when
represented in UCS2.


See http://www.w3.org/2011/rdf-wg/wiki/Turtle_Test_Suite for more details.


Eric Prud'hommeaux <eric+turtle@w3.org> - 11 June 2013.
