This README is for the W3C RDF Working Group's N-Triples test suite.
This test suite contains two kinds of tests:

  Positive syntax (rdft:TestNTriplesPositiveSyntax) - an input N-Triples
  file with no syntax errors.

  Negative syntax (rdft:TestNTriplesNegativeSyntax) - an input N-Triples
  file with at least one syntax error.

The manifest.ttl file in this directory lists tests in the
RDF WG's N-Triples test suite. All
tests have a name (mf:name) and an input (mf:action).

• An implementation passes a positive syntax test if it parses the
  input.

• An implementation passes a negative syntax test if it fails to parse
  the input.

The home of the test suite is <http://www.w3.org/2013/N-TriplesTests/>.

See http://www.w3.org/2011/rdf-wg/wiki/RDF_Test_Suites for more details.

Eric Prud'hommeaux <eric+turtle@w3.org> - 11 June 2013.
Gregg Kellogg <gregg@greggkellogg.net> - 26 June 2013.
