# Non-normative tests W3C tests for RDF/XML

This test suite contains non-normative tests for RDF/XML. These tests originate
from tests commented out in the [W3C RDF/XML test
suite](http://www.w3.org/2013/RDFXMLTests/).

These tests are not added in an RDFLib specific test suite as they are authored
under W3C efforts.

The original home of the tests in this suite is
<http://www.w3.org/2013/RDFXMLTests/>. Per RFC 3986 section 5.1.3, the base IRI
for parsing each file is the retrieval IRI for that file. For example, the
tests rdfms-xml-literal-namespaces-test001 require relative IRI resolution
against a base of
<http://www.w3.org/2013/RDFXMLTests/rdfms-xml-literal-namespaces/test001.rdf>.

For more information see:
- <http://www.w3.org/2013/RDFXMLTests/README>.
- <https://github.com/w3c/rdf-tests/issues/80>.
